/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */


/*
 * Clustering.hpp
 *
 *  Created on: 2010/12/16
 *      Author: HARK Development Team
 */

#ifndef CLUSTERING_HPP_
#define CLUSTERING_HPP_

#include <vector>
#include <boost/shared_ptr.hpp>
#include <Cluster.hpp>
#include <Band.hpp>
#include <Signal.hpp>
#include <Peak.hpp>

namespace NSHarkBinaural
{

class Clustering
{
public:
    typedef boost::shared_ptr<Cluster> ClusterPtr;
    typedef std::vector<ClusterPtr> ClusterVector;
    typedef boost::shared_ptr<ClusterVector> ClusterVectorPtr;
    typedef std::vector<Band> BandVector;
    typedef boost::shared_ptr<BandVector> BandVectorPtr;

    /**
     * 信号のピークをクラスタリングする.
     * 与えられたピークのリストの中で、パワーがminDB以上のものを対象として
     * クラスタリングし、結果を返す。
     * @param peak 信号のピーク一覧
     * @param maxCost クラスタリングするときのコストの最大値
     * @return クラスタリング結果
     */
    static ClusterVectorPtr
    Execute(Peak::PeakVectorPtr peak, double maxCost = 5.0);

    /*
     * 戻り値の違いによる速度についてメモ。
     * vector<hoge>で戻り値を返す場合で、RVO(Return Value Optimization)が無効のとき
     *  1. 関数内でvector<hoge>がスタック上に生成される
     *  2. returnするとき、スタックの戻り値領域にコピーされる
     *  3. 関数呼び出し側で vector<hoge> result = foo(); などとなっていると、
     *     スタックの戻り値領域からresultへコピーされる
     *  コピー回数は2回
     * vector<hoge>で戻り値を返す場合で、RVO(Return Value Optimization)が有効のとき
     *  1. 関数内でvector<hoge>がスタック上（ただし、普通の変数と同じ場所ではなく、戻り値領域である。）に生成される
     *  2. returnするとき、スタックの戻り値領域にコピーされない（元々戻り値領域にあるから。）
     *  3. 関数呼び出し側で vector<hoge> result = foo(); などとなっていても、コピーされない
     *     resultの実態は関数内で生成した変数そのものである（つまりresultのアドレスは戻り値領域のアドレスになっている）
     *  コピー回数は0回
     * shared_ptr<vector<hoge> >で戻り値を返す場合、
     *   1. shared_ptrに内包されるvectorオブジェクトのコピーは、RVOが有効でも無効でも発生しない（あたりまえだが。）
     *   2. shared_ptr自体のコピーが行われる（これはRVOの有効無効で挙動が変わる）
     *      コピーが2回発生しても、shared_ptr自体のコピーに要するコストは低いので、
     *      vector自体がコピーされる場合に比べて影響はほとんどない
     * 以上から、RVOの有効無効で速度が安定するのがshared_ptrによる戻り値で、
     * RVOが有効のときに最高効率になるが、RVOが無効だと著しく遅くなるのがvectorによる戻り値である。
     */

    /**
     * 各クラスタを代表する周波数バンドのリストを得る.
     * クラスタ1つにつき、1つ代表の周波数バンドを選ぶ。
     * これをすべてのクラスタについて行い、そのリストを返す。
     * @param clusters クラスタのリスト
     * @param bgnoise バックグラウンドノイズのパワースペクトル
     * @return 代表の周波数バンドのリスト（インデックスはclustersのインデックスに対応する）
     */
    static BandVectorPtr
    GetProxy(ClusterVectorPtr clusters, Unspectrum::ptr_t bgnoise);

private:
    /**
     * 2つのクラスタのコストとクラスタ番号を表す.
     */
    struct CostAndIndex
    {
        double cost;
        size_t index1, index2;
    };

    /**
     * 初期クラスタ集合を生成する.
     * 各クラスタは、一つだけ周波数バンドを含む状態になる。
     */
    static ClusterVectorPtr
    MakeInitClusters(Peak::PeakVectorPtr peak);

    /**
     * もっとも近いクラスタの組を見つける.
     */
    static CostAndIndex
    GetClosestDst(ClusterVectorPtr clusters);
};

}

#endif /* CLUSTERING_HPP_ */
