/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */


/*
 * EventHistory.hpp
 *
 *  Created on: 2011/01/24
 *      Author: HARK Development Team
 */

#ifndef EVENTHISTORY_HPP_
#define EVENTHISTORY_HPP_

#include <sys/time.h>
#include <DimensionPromise/types/Angle.hpp>
#include <EventID.hpp>
#include <EventStatus.hpp>
#include <Band.hpp>
#include <Logger.hpp>

namespace NSHarkBinaural
{


class EventHistory
{
public:
    /**
     * イベントを生成する.
     * @param eid イベントID
     * @param currentTime イベント生成時刻
     * @param validPausePeriod PAUSE状態が続いてもイベントが有効のままである期間
     * @param fundamental 基音の初期値
     * @param direction 方向の初期値
     * @param updateRatioThreshold 基音・角度付きで更新した頻度（回数/秒）の閾値（これ以下になるとイベントが終了する）
     */
    EventHistory(EventID eid, const timeval& currentTime, const timeval& validPausePeriod, Band fundamental, NSDimensionPromise::Angle direction, double updateRatioThreshold);

    /**
     * イベントを更新する.
     * 入力信号が存在する場合のメソッド。イベント状態をDURATIONにする。
     * @param currentTime 現在時刻
     * @param fundamental 新しい基音
     * @param direction 新しい方向
     */
    void Update(const timeval& currentTime, Band fundamental, NSDimensionPromise::Angle direction);

    /**
     * イベントを更新する.
     * 入力信号が無い場合のメソッド。イベント状態をPAUSEにする。
     * @param currentTime 現在時刻
     */
    void Update(const timeval& currentTime);

    /**
     * イベントが終了したかどうか.
     */
    bool IsOutOfDate() const {
        return this->status == EventStatus::OFFSET;
    }

    /**
     * イベントIDを得る.
     */
    const EventID& ID() const {
        return this->eid;
    }

    /**
     * 現在のイベントの状態を得る.
     */
    EventStatus::Kind Status() const {
        return this->status;
    }

    /**
     * 最後に設定された基音を得る.
     */
    const Band& Fundamental() const {
        return this->fundamental;
    }

    /**
     * 最後に設定された方向を得る.
     */
    NSDimensionPromise::Angle Direction() const {
        return this->direction;
    }

    /**
     * イベントが最後にPAUSEとなった時刻を得る.
     */
    const timeval& PauseTime() const {
        return this->pauseTime;
    }

    /**
     * イベントが最後にDURATIONとなった時刻を得る.
     */
    const timeval& DurationTime() const {
        return this->pauseTime;
    }

    /**
     * イベントが生成された時刻を得る.
     */
    const timeval& OnsetTime() const {
        return this->onsetTime;
    }

private:

    /// イベントID
    EventID eid;

    /// イベントの状態
    EventStatus::Kind status;

    /// 基底周波数バンド
    Band fundamental;

    /// 音の方向
    NSDimensionPromise::Angle direction;

    /// statusが最後にPAUSEとなった時刻
    timeval pauseTime;

    /// statusが最後にDURATIONとなった時刻
    timeval durationTime;

    /// イベント生成時刻
    const timeval onsetTime;

    /// PAUSE状態が続いてもイベントが有効のままである期間
    const timeval& validPausePeriod;

    /**
     * このイベントが角度と基音を指定してアップデートされた回数.
     * このイベントが
     * void Update(const timeval& currentTime, Band fundamental, NSDimensionPromise::Angle direction);
     * によってアップデートされた回数。
     */
    size_t updateCount;

    /**
     * このイベントが角度と基音を指定してアップデートされた頻度の閾値.
     * (updateCount / このイベントが生成されてからの時間) <= updateRatioThreshold
     * となると、イベントが終了する。
     */
    double updateRatioThreshold;

    /// ロガー
    Logger& logger;

    /// アップデート頻度をチェックし、閾値以下だったらイベントを終了させる.
    void CheckUpdateRatio(const timeval& currentTime);
};

}

#endif /* EVENTHISTORY_HPP_ */
