/*
 * Copyright 2008 Kyoto University and Honda Motor Co.,Ltd.
 * All rights reserved.
 * HARK was developed by researchers in Okuno Laboratory
 * at the Kyoto University and Honda Research Institute Japan Co.,Ltd.
 */


/*
 * Separation.hpp
 *
 *  Created on: 2010/11/11
 *      Author: HARK Development Team
 */

/**
 * @file Separation.hpp
 * 音源分離を行うクラス群.
 */

#ifndef SEPARATION_HPP_
#define SEPARATION_HPP_

#include <boost/shared_ptr.hpp>

#include <DimensionPromise/types.hpp>

#include <SeparationSetting.hpp>
#include <Signal.hpp>
#include <Channel.hpp>
#include <LRSignal.hpp>
#include <IIPDS.hpp>

namespace NSHarkBinaural
{

/**
 * 音源分離ベースクラス.
 */
class Separation
{
public:
    /**
     * 左右両チャンネルのスペクトルを表す型.
     */
    typedef ChannelHolder<Spectrum::ptr_t, Spectrum::ptr_t> LRSpectra;

    /**
     * スマートポインタ型
     */
    typedef boost::shared_ptr<Separation> ptr_t;

    /**
     * 音源分離ベースクラスを生成する.
     * @param inputSpectra 分離対象の音源のスペクトル（左右両チャンネルのスペクトル）
     * @param separationSetting 分離の設定
     */
    Separation(LRSpectra::ptr_t inputSpectra, SeparationSetting::ptr_t separationSetting)
        : inputSpectra(inputSpectra), separationSetting(separationSetting),
          logger(Logger::GetLogger(Logger::GLOBAL_LOGGER_NAME))
    {
        LogFine(logger, "Separation::Separation");
    }

    /**
     * 音源分離ベースクラスを破棄する.
     */
    virtual ~Separation()
    {
        LogFine(logger, "Separation::~Separation");
    }

    /**
     * 音源を分離する.
     * 指定された方向の、指定された角度範囲にある音を分離し、返す
     * @param dir 分離する方向
     * @param range 分離する角度範囲
     * @return 分離されたスペクトル
     */
    virtual LRSpectra::ptr_t separate(NSDimensionPromise::Angle dir, NSDimensionPromise::Angle range) = 0;

protected:
    /**
     * ロガー.
     */
    Logger& logger;

    /**
     * 入力信号のスペクトルを得る.
     * @return 入力信号のスペクトル
     */
    LRSpectra::ptr_t GetInputSpectra()
    {
        return this->inputSpectra;
    }

    /**
     * 分離の設定を得る.
     * @return 分離の設定
     */
    SeparationSetting::ptr_t GetSeparationSetting()
    {
        return this->separationSetting;
    }

private:
    /**
     * 分離対象の音源のスペクトル.
     * 分離帯小の音源の、左右のスペクトルデータ
     */
    LRSpectra::ptr_t inputSpectra;

    /**
     * 分離の設定.
     */
    SeparationSetting::ptr_t separationSetting;
};

/**
 * Epipolarによる音源分離クラス.
 */
class EpipolarSeparation : public Separation
{
public:
    /**
     * 音源分離クラスを生成する.
     * @param inputSpectra 分離対象の音源のスペクトル（左右両チャンネルのスペクトル）
     * @param separationSetting 分離の設定
     */
    EpipolarSeparation(LRSpectra::ptr_t inputSpectra, SeparationSetting::ptr_t separationSetting)
        : Separation(inputSpectra, separationSetting)
    {
        LogFine(logger, "EpipolarSeparation::EpipolarSeparation");
    }

    /**
     * スマートポインタ型
     */
    typedef boost::shared_ptr<EpipolarSeparation> ptr_t;

    /**
     * 音源分離クラスを破棄する.
     */
    virtual ~EpipolarSeparation()
    {
        LogFine(logger, "EpipolarSeparation::~EpipolarSeparation");
    }

    /**
     * 音源を分離する.
     * 指定された方向の、指定された角度範囲にある音を分離し、返す
     * @param dir 分離する方向
     * @param range 分離する角度範囲
     * @return 分離されたスペクトル
     */
    virtual LRSpectra::ptr_t separate(NSDimensionPromise::Angle dir, NSDimensionPromise::Angle range);
};


/**
 * HRTFによる音源分離クラス.
 */
class HRTFSeparation : public Separation
{
public:
    /**
     * 音源分離クラスを生成する.
     * @param inputSpectra 分離対象の音源のスペクトル（左右両チャンネルのスペクトル）
     * @param separationSetting 分離の設定
     * @param bgnoise ルームノイズのパワースペクトル（デシベル値）
     * @param iipds 頭部伝達関数
     * @param thresDB ルームノイズに対する信号パワーの閾値
     */
    HRTFSeparation(LRSpectra::ptr_t inputSpectra, SeparationSetting::ptr_t separationSetting,
                   LRUnspectra::ptr_t bgnoise, IIPDS& iipds, Unspectrum::ElementType thresDB)
        : Separation(inputSpectra, separationSetting), bgnoise(bgnoise), iipds(iipds), thresDB(thresDB)
    {

    }

    /**
     * 音源を分離する.
     * 指定された方向の、指定された角度範囲にある音を分離し、返す
     * @param dir 分離する方向
     * @param range 分離する角度範囲
     * @return 分離されたスペクトル
     */
    virtual LRSpectra::ptr_t separate(NSDimensionPromise::Angle dir, NSDimensionPromise::Angle range);

private:
    /**
     * ルームノイズ.
     */
    LRUnspectra::ptr_t bgnoise;

    /**
     * 頭部伝達関数.
     */
    IIPDS& iipds;

    /**
     * ルームノイズに対する信号パワーの閾値.
     */
    Unspectrum::ElementType thresDB;
};

/**
 * 角度ごとに許容される分離の範囲を取得する.
 *  90度 -> max
 *   0度 -> min
 * となる線形関数
 */
NSDimensionPromise::Angle getPermissibleDirectionRange(
    NSDimensionPromise::Angle dir,
    NSDimensionPromise::Angle min = NSDimensionPromise::Angle::FromDegree(5),
    NSDimensionPromise::Angle max = NSDimensionPromise::Angle::FromDegree(30));

}

#endif /* SEPARATION_HPP_ */
